class Displayer:
    def __str__(self):
        res = '%s\n' % self.__class__.__name__
        for attr in sorted(self.__dict__.keys()):
            res += '\t%s=%s\n' % (attr, self.__dict__[attr])            
        return res

class Contact(Displayer):
    """
    contact records class
    """
    def __init__(self, name, phone, notes=None):
        self.name = name
        self.phone = phone
        self.notes = [] if notes == None else notes
    def addNote(self, note):
        """
        insert a new note on object
        """
        self.notes.append(note)
    def areaCode(self):
        """
        strip off area code at front
        """
        return self.phone.split('-')[0]

if __name__ == '__main__':
    c1 = Contact('Bob Smith', '303-123-4567', ['coworker'])
    c2 = Contact('Sue Jones', '970-867-5309')
    print c1
    print c2
    print c1.areaCode(), c2.areaCode()
    c2.addNote('manager')
    c1.addNote('engineer')
    print c1





############################################################
# related interactive text
############################################################


# output of self-test code
"""
>>> 
Contact
	name=Bob Smith
	notes=['coworker']
	phone=303-123-4567

Contact
	name=Sue Jones
	notes=[]
	phone=970-867-5309

303 970
Contact
	name=Bob Smith
	notes=['coworker', 'engineer']
	phone=303-123-4567
"""



# mixing diaplyer into another class
"""
>>> from contact import Displayer
>>> 
>>> class C(Displayer):
	def spam(self):
		return 42
	
>>> X = C()
>>> X.name = 'bob'
>>> X.age = 40
>>> X.job = 'dev'
>>> print X
C:
job => dev
age => 40
name => bob
"""



# inspecting the __dict__ attribute, getattr and dir inherit
"""
>>> class C:
	def spam(self):
		return 42

>>> C.__dict__
{'__module__': '__main__', '__doc__': None, 'spam': <function spam at 0x02A865B0>}
>>> 
>>> X = C()
>>> X.__dict__
{}
>>> getattr(X, 'spam')
<bound method C.spam of <__main__.C instance at 0x02A9AD00>>
>>> 
>>> X.job  = 'dev'
>>> 
>>> X.__dict__
{'job': 'dev'}
>>> 
>>> getattr(X, 'spam')
<bound method C.spam of <__main__.C instance at 0x02A9AD00>>
>>> 
>>> dir(X)
['__doc__', '__module__', 'job', 'spam']


# additional interaction

>>> class C:
	def __init__(s):
		s.a = 1
		s.b = 2
		s.c = 3
		
>>> X = C()
>>> X.a
1
>>> X.b
2
>>> X.c
3
>>> 
>>> X.__dict__
{'a': 1, 'c': 3, 'b': 2}
>>> 
>>> for attr in X.__dict__.keys():
	print attr, X.__dict__[attr]
	
a 1
c 3
b 2
>>> 
>>> X.a
1
>>> X.__dict__['a']
1
>>> getattr(X, 'a')
1
"""
